/*
 * TismTool © Tis Veugen, 2010-2011
 * Application
 *
 * TsmQueue.h
 * version 3
 */
#ifndef TSM_QUEUE_H
#define TSM_QUEUE_H

#include <deque>
#include <algorithm>
#include <boost/thread/mutex.hpp>
#include <boost/thread/locks.hpp>
#include "TsmEnvelope.h"
using namespace std;

namespace TsmRTE
{
  class TsmQueue : public deque<TsmEnvelope>
  {
  private:
    unsigned int  mMaxQueueSize;
    boost::mutex  mLockMutex;

  public:
    TsmQueue()
      : mMaxQueueSize(0)
      , mLockMutex()
    {
    }

    int getMaxQueueSize()
    {
      return mMaxQueueSize;
    }

    void Put( TsmEnvelope& oEnvelope )
    {
      boost::unique_lock<boost::mutex> lock( mLockMutex );

      this->push_back( oEnvelope );
      if ( mMaxQueueSize < this->size() )
      {
        mMaxQueueSize = this->size();
      }
    }

    bool Dequeue( TsmEnvelope& oEnvelope )
    {
      boost::unique_lock<boost::mutex> lock( mLockMutex );

      if ( this->empty() )
      {
        return false;
      }

      oEnvelope = static_cast<TsmEnvelope>( this->front() );
      this->pop_front();
      return true;
    }

    string Snap()
    {
      deque<TsmEnvelope>::iterator it = this->begin();
      stringstream ss;

      while ( it != this->end() )
      {
        ss << (*it).Snap() + ",";
        it++;
      }
      return ss.str();
    }
  };
}
#endif
