/*
 * TismTool © Tis Veugen, 2010-2011
 * Application
 *
 * TsmThread.cpp
 * version 5
 */
#include <boost/thread/mutex.hpp>    
#include <boost/thread/locks.hpp>
#include <boost/thread/thread.hpp>
#include "TsmThread.h"
#include "TsmSupervisor.h"
using namespace std;

namespace TsmRTE
{
  TsmThread::TsmThread( TsmThreadType Type )
    : mType( Type )
  {
    mExternalQueue = new TsmExtQueue();
    mInternalQueue = new TsmIntQueue();
    mExclusiveMutex = new TsmMutex();
  }

  TsmThread::~TsmThread()
  {
    delete mThread;
    delete mInternalQueue;
    delete mExternalQueue;
    delete mExclusiveMutex;
  }

  void TsmThread::Prepare( string sName )
  {
    mbHalt = false;
    mThread = new boost::thread( Worker, this );
    mSequence = TsmSupervisor::Register( this, mType );
    stringstream ss;
    ss << "Tism" << mSequence << "_" << sName;
    this->setName( ss.str() );

    TsmSupervisor::Announce();
  }

  void TsmThread::Halt()
  {
    mbHalt = true;
    TriggerNotify();
    mThread->join();
  }

  void TsmThread::ASyncCall( TsmEnvelope& oEnvelope )
  {
    oEnvelope.getTrigger()->SetDBU();
    mExternalQueue->Put( oEnvelope );

    TriggerNotify();
  }

  void TsmThread::SyncCall( TsmEnvelope& oEnvelope )
  {
    ExclusiveLock();

    oEnvelope.getTrigger()->RvState = WAIT;
    mExternalQueue->Put( oEnvelope );

    TriggerNotify();
    RendezVousWait();

    ExclusiveUnlock();
  }

  TsmRvState TsmThread::Handle( TsmEnvelope& oEnvelope )
  {
    TsmTrigger* lTrigger = oEnvelope.getTrigger();

    oEnvelope.getTarget()->ExecuteTrigger( lTrigger );
    if ( lTrigger->RvState == READY )
    {
      RendezVousNotify();
      lTrigger->RvState = IDLE;
    }
    return lTrigger->RvState;
  }

  void Worker( TsmThread* oTsmThread )
  {
    TsmEnvelope   lEnvelope;
    TsmRvState    lRvState = IDLE;
    bool          bMsg = false;

    oTsmThread->RendezVousWait();

    while ( ! oTsmThread->mbHalt )
    {
      oTsmThread->TriggerWait();

      do
      {
        bMsg = false;

        if ( oTsmThread->mInternalQueue->Dequeue( lEnvelope ) )
        {
          bMsg = true;
          lEnvelope.getTrigger()->RvState = lRvState;

          try
          {
            lRvState = oTsmThread->Handle( lEnvelope );
          }
          catch ( exception& e )
          {
            Tsm::ShowException( "@S", e );
          }
          lEnvelope.getTrigger()->DBU();
        }
        else
        {
          if ( oTsmThread->mExternalQueue->Dequeue( lEnvelope ) )
          {
            bMsg = true;
            if ( lEnvelope.getTrigger()->RvState == WAIT )
            {
              lRvState = WAIT;
            }
            else
            {
              lEnvelope.getTrigger()->RvState = lRvState;
            }

            try
            {
              lRvState = oTsmThread->Handle( lEnvelope );
            }
            catch ( exception& e )
            {
              Tsm::ShowException( "@Q", e );
            }
            lEnvelope.getTrigger()->DBU();
          }
        }
      } while ( bMsg );
    }
  }
}
