/*
 * TismTool © Tis Veugen, 2010-2011
 * Application
 *
 * TsmThread.h
 * version 3
 */
#ifndef TSM_THREAD_H
#define TSM_THREAD_H

#include <boost/thread/mutex.hpp>    
#include <boost/thread/locks.hpp>
#include <boost/thread/thread.hpp>

#include "TsmThreadType.h"
#include "TsmExtQueue.h"
#include "TsmIntQueue.h"
#include "TsmMutex.h"
#include "TsmRzVousSem.h"
#include "TsmTriggerSem.h"
#include "TsmEnvelope.h"
#include "TsmRvState.h"
using namespace std;

namespace TsmRTE
{
  class TsmThread
  {
  public:
    TsmThreadType   mType;
    int             mSequence;
    bool            mbHalt;
    TsmExtQueue*    mExternalQueue;
    TsmIntQueue*    mInternalQueue;

  protected:
    boost::thread* mThread;
    TsmMutex*      mExclusiveMutex;
    TsmRzVousSem   mRendezVousSem;
    TsmTriggerSem  mTriggerSem;
    boost::mutex   mLockMutex;
    string         mName;

  public:
    TsmThread( TsmThreadType Type = THREAD );
    virtual ~TsmThread();

    boost::thread* getThread()
    {
      return mThread;
    }

    TsmIntQueue* getIntQueue()
    {
      return mInternalQueue;
    }

    bool hasMessage()
    {
      boost::unique_lock<boost::mutex> lock( mLockMutex );
      return ( mExternalQueue->size() + mInternalQueue->size() > 0 );
    }

    virtual void Prepare( string sName );

    virtual void Prepare()
    {
    }

    virtual void Run()
    {
      RendezVousNotify();
    }

    virtual void Halt();

    virtual void Interrupt()
    {
      mThread->interrupt();
    }

    virtual void Cease()
    {
    }

    void ExclusiveLock()
    {
      mExclusiveMutex->WaitOne();
    }

    void ExclusiveUnlock()
    {
      mExclusiveMutex->ReleaseMutex();
    }

    void RendezVousWait()
    {
      mRendezVousSem.WaitOne();
    }

    void RendezVousNotify()
    {
      mRendezVousSem.Release();
    }

    void TriggerWait()
    {
      mTriggerSem.WaitOne();
    }

    void TriggerNotify()
    {
      mTriggerSem.Release();
    }

    void ASyncCall( TsmEnvelope& oEnvelope );
    void SyncCall( TsmEnvelope& oEnvelope );
    TsmRvState Handle( TsmEnvelope& oEnvelope );

    virtual void NativeCall( TsmEnvelope& oEnvelope )
    {
      // To be overridden!
    }

    virtual void CbHandle( TsmEnvelope& oEnvelope )
    {
      // To be overridden!
    }

    string getName()
    {
      return mName;
    }

    void setName( string s )
    {
      mName = s;
    }
  };

  void Worker( TsmThread* oTsmThread );

}
#endif
